@@ -1,17 +1,15 @@
 from s2protocol import versions
 from mpyq import MPQArchive
-import os
-
 from typing import List
 
 
 class SC2ReplayParser(object):
     def __init__(self, replay: str):
-        assert os.path.isfile(replay), f'{replay} is not a file'
         self.archive = MPQArchive(replay)
         self.protocol = self.read_protocol(self.archive)
         self._initdata = None
         self._details = None
+        self._events = None
 
     @staticmethod
     def read_protocol(archive: MPQArchive):
@@ -30,6 +28,11 @@ class SC2ReplayParser(object):
         detail_file = archive.read_file('replay.details')
         return protocol.decode_replay_details(detail_file)
 
+    @staticmethod
+    def read_game_events(protocol, archive: MPQArchive) -> dict:
+        game_event_file = archive.read_file('replay.game.events')
+        return protocol.decode_replay_game_events(game_event_file)
+
     def initdata(self) -> dict:
         if self._initdata is None:
             self._initdata = self.read_initdata(self.protocol, self.archive)
@@ -41,6 +44,11 @@ class SC2ReplayParser(object):
             self._details = self.read_details(self.protocol, self.archive)
         return self._details
 
+    def events(self):
+        if self._events is None:
+            self._events = list(self.read_game_events(self.protocol, self.archive))
+        return self._events
+
     def players(self) -> List[dict]:
         slots = self.initdata()['m_syncLobbyState']['m_lobbyState']['m_slots']
         slot_id_to_user_id = {}
@@ -55,6 +63,8 @@ class SC2ReplayParser(object):
             if slot_id in slot_id_to_user_id:
                 player['m_userId'] = slot_id_to_user_id[slot_id]
                 player['m_userInitialData'] = self.initdata()['m_syncLobbyState']['m_userInitialData'][slot_id]
+                player['m_localizedId'] = '{}/{}/{}'.format(player['m_toon']['m_region'], player['m_toon']['m_realm'],
+                                                            player['m_toon']['m_id'])
                 player_objects.append(player)
         return player_objects
 
@@ -71,7 +81,29 @@ class SC2ReplayParser(object):
         }
 
     def to_dict(self):
-        return {
+        payload = {
             'details': self.small_details(),
             'players': self.players()
         }
+        return decode_utf8(payload)
+
+
+# Slightly modified from:
+# https://stackoverflow.com/questions/57014259/json-dumps-on-dictionary-with-bytes-for-keys
+def decode_utf8(d: dict) -> dict:
+    result = {}
+    for key, value in d.items():
+        try:
+            if isinstance(key, bytes):
+                key = key.decode()
+            if isinstance(value, bytes):
+                value = value.decode()
+            elif isinstance(value, dict):
+                value = decode_utf8(value)
+            elif isinstance(value, list):
+                decoded = [decode_utf8(v) for v in value]
+                value = decoded
+        except:
+            value = ''
+        result.update({key: value})
+    return result
